pub enum MockChainNote {
Private(NoteId, NoteMetadata, NoteInclusionProof),
Public(Note, NoteInclusionProof),
}Expand description
Represents a note that is stored in the mock chain.
Variants
Private(NoteId, NoteMetadata, NoteInclusionProof)
Details for a private note only include its NoteMetadata and NoteInclusionProof.
Other details needed to consume the note are expected to be stored locally, off-chain.
Public(Note, NoteInclusionProof)
Contains the full Note object alongside its NoteInclusionProof.
Implementations
Sourceimpl MockChainNote
impl MockChainNote
Sourcepub fn inclusion_proof(&self) -> &NoteInclusionProof
pub fn inclusion_proof(&self) -> &NoteInclusionProof
Returns the note’s inclusion details.
Sourcepub fn metadata(&self) -> &NoteMetadata
pub fn metadata(&self) -> &NoteMetadata
Returns the note’s metadata.