pub struct TransactionContext { /* private fields */ }Expand description
Represents all needed data for executing a transaction, or arbitrary code.
It implements DataStore, so transactions may be executed with
TransactionExecutor
Implementations
Sourceimpl TransactionContext
impl TransactionContext
Sourcepub async fn execute_code(
&self,
code: &str,
) -> Result<ExecutionOutput, ExecutionError>
pub async fn execute_code( &self, code: &str, ) -> Result<ExecutionOutput, ExecutionError>
Executes arbitrary code within the context of a mocked transaction environment and returns
the resulting ExecutionOutput.
The code is compiled with the assembler built by CodeBuilder::with_mock_libraries
and executed with advice inputs constructed from the data stored in the context. The program
is run on a modified TransactionExecutorHost which is loaded with the procedures exposed
by the transaction kernel, and also individual kernel functions (not normally exposed).
To improve the error message quality, convert the returned ExecutionError into a
Report or use ? with
miden_protocol::assembly::diagnostics::Result.
Errors
Returns an error if the assembly or execution of the provided code fails.
Panics
- If the provided
codeis not a valid program.
Sourcepub async fn execute(
self,
) -> Result<ExecutedTransaction, TransactionExecutorError>
pub async fn execute( self, ) -> Result<ExecutedTransaction, TransactionExecutorError>
Executes the transaction through a TransactionExecutor
pub fn account(&self) -> &Account
pub fn expected_output_notes(&self) -> &[Note]
pub fn tx_args(&self) -> &TransactionArgs
pub fn input_notes(&self) -> &InputNotes<InputNote>
pub fn set_tx_args(&mut self, tx_args: TransactionArgs)
pub fn tx_inputs(&self) -> &TransactionInputs
pub fn authenticator(&self) -> Option<&BasicAuthenticator>
Sourcepub fn source_manager(&self) -> Arc<dyn SourceManagerSync>
pub fn source_manager(&self) -> Arc<dyn SourceManagerSync>
Returns the source manager used in the assembler of the transaction context builder.