pub struct TransactionContextBuilder { /* private fields */ }Expand description
TransactionContextBuilder is a utility to construct TransactionContext for testing purposes. It allows users to build accounts, create notes, provide advice inputs, and execute code. The VM process can be inspected afterward.
Examples
Create a transaction context for an existing account and execute code:
let mut builder = MockChain::builder();
let account = builder.add_existing_mock_account(Auth::IncrNonce)?;
let mock_chain = builder.build()?;
let tx_context = mock_chain.build_tx_context(account.id(), &[], &[])?.build()?;
let code = "
use miden::tx_kernel_core::prologue
begin
exec.prologue::prepare_transaction
push.5
swap drop
end
";
let exec_output = tx_context.execute_code(code).await?;
assert_eq!(exec_output.stack.get(0).unwrap(), &Felt::from(5u32));Implementations
Sourceimpl TransactionContextBuilder
impl TransactionContextBuilder
pub fn new(account: Account) -> Self
Sourcepub fn extend_advice_inputs(self, advice_inputs: AdviceInputs) -> Self
pub fn extend_advice_inputs(self, advice_inputs: AdviceInputs) -> Self
Extend the advice inputs with the provided AdviceInputs instance.
Sourcepub fn extend_advice_map(
self,
map_entries: impl IntoIterator<Item = (Word, Vec<Felt>)>,
) -> Self
pub fn extend_advice_map( self, map_entries: impl IntoIterator<Item = (Word, Vec<Felt>)>, ) -> Self
Extend the advice inputs map with the provided iterator.
Sourcepub fn authenticator(self, authenticator: Option<BasicAuthenticator>) -> Self
pub fn authenticator(self, authenticator: Option<BasicAuthenticator>) -> Self
Set the authenticator for the transaction (if needed)
Sourcepub fn foreign_accounts(
self,
inputs: impl IntoIterator<Item = (Account, AccountWitness)>,
) -> Self
pub fn foreign_accounts( self, inputs: impl IntoIterator<Item = (Account, AccountWitness)>, ) -> Self
Set foreign account codes that are used by the transaction
Sourcepub fn tx_script(self, tx_script: TransactionScript) -> Self
pub fn tx_script(self, tx_script: TransactionScript) -> Self
Set the desired transaction script
Sourcepub fn tx_script_args(self, tx_script_args: Word) -> Self
pub fn tx_script_args(self, tx_script_args: Word) -> Self
Set the transaction script arguments
Sourcepub fn tx_inputs(self, tx_inputs: TransactionInputs) -> Self
pub fn tx_inputs(self, tx_inputs: TransactionInputs) -> Self
Set the desired transaction inputs
Sourcepub fn disable_lazy_loading(self) -> Self
pub fn disable_lazy_loading(self) -> Self
Disables lazy loading.
Only affects TransactionContext::execute_code and causes the host to not handle lazy
loading events.
Sourcepub fn extend_note_args(self, note_args: BTreeMap<NoteId, Word>) -> Self
pub fn extend_note_args(self, note_args: BTreeMap<NoteId, Word>) -> Self
Extend the note arguments map with the provided one.
Sourcepub fn extend_expected_output_notes(
self,
output_notes: Vec<RawOutputNote>,
) -> Self
pub fn extend_expected_output_notes( self, output_notes: Vec<RawOutputNote>, ) -> Self
Extend the expected output notes.
Sourcepub fn with_source_manager(
self,
source_manager: Arc<dyn SourceManagerSync>,
) -> Self
pub fn with_source_manager( self, source_manager: Arc<dyn SourceManagerSync>, ) -> Self
Sets the SourceManagerSync on the TransactionContext that will be built.
This source manager should contain the sources of all involved scripts and account code in order to provide better error messages if an error occurs.
Sourcepub fn add_signature(
self,
pub_key: PublicKeyCommitment,
message: Word,
signature: Signature,
) -> Self
pub fn add_signature( self, pub_key: PublicKeyCommitment, message: Word, signature: Signature, ) -> Self
Add a new signature for the message and the public key.
Sourcepub fn add_note_script(self, script: NoteScript) -> Self
pub fn add_note_script(self, script: NoteScript) -> Self
Add a note script to the context for testing.
Sourcepub fn build(self) -> Result<TransactionContext>
pub fn build(self) -> Result<TransactionContext>
Builds the TransactionContext.
If no transaction inputs were provided manually, an ad-hoc MockChain is created in order to generate valid block data for the required notes.