create_p2any_note

Function create_p2any_note 

pub fn create_p2any_note(
    sender: AccountId,
    note_type: NoteType,
    assets: impl IntoIterator<Item = Asset>,
    rng: &mut RpoRandomCoin,
) -> Note
Creates a P2ANY note.

A P2ANY note carries assets and a script that moves the assets into the executing account’s vault.

The created note does not require authentication and can be consumed by any account.