pub enum Auth {
BasicAuth {
auth_scheme: AuthScheme,
},
Multisig {
approver_set: ApproverSet,
proc_threshold_map: Vec<(AccountProcedureRoot, u32)>,
},
GuardedMultisig {
approver_set: ApproverSet,
guardian_config: GuardianConfig,
proc_threshold_map: Vec<(AccountProcedureRoot, u32)>,
},
MultisigSmart {
approver_set: ApproverSet,
proc_policy_map: Vec<(Word, ProcedurePolicy)>,
},
Acl {
exempt_procedures: BTreeSet<AccountProcedureRoot>,
auth_scheme: AuthScheme,
},
IncrNonce,
Noop,
Conditional,
NetworkAccount {
allowed_script_roots: BTreeSet<NoteScriptRoot>,
allowed_tx_script_roots: BTreeSet<TransactionScriptRoot>,
},
}Expand description
Specifies which authentication mechanism is desired for accounts
Variants
BasicAuth
Creates a secret key for the account and creates a BasicAuthenticator used to authenticate the account with AuthSingleSig.
Fields
auth_scheme: AuthSchemeMultisig
Multisig
GuardedMultisig
Guarded multisig.
Fields
approver_set: ApproverSetguardian_config: GuardianConfigproc_threshold_map: Vec<(AccountProcedureRoot, u32)>MultisigSmart
Multisig with smart per-procedure policy configuration.
Acl
Creates a secret key for the account, and creates a BasicAuthenticator used to
authenticate the account with AuthSingleSigAcl. Any called procedure that is not
in exempt_procedures forces signature verification.
IncrNonce
Creates a mock authentication mechanism for the account that only increments the nonce.
Noop
Creates a mock authentication mechanism for the account that does nothing.
Conditional
Creates a mock authentication mechanism for the account that conditionally succeeds and conditionally increments the nonce based on the authentication arguments.
The auth procedure expects the first three arguments as [99, 98, 97] to succeed. In case it succeeds, it conditionally increments the nonce based on the fourth argument.
NetworkAccount
Network-account authentication that restricts the account to consuming only notes whose
script roots appear in allowed_script_roots (must be non-empty), and to executing only
transaction scripts whose roots appear in allowed_tx_script_roots (may be empty).
Fields
allowed_script_roots: BTreeSet<NoteScriptRoot>allowed_tx_script_roots: BTreeSet<TransactionScriptRoot>Implementations
Sourceimpl Auth
impl Auth
Sourcepub fn build_component(&self) -> (AccountComponent, Option<BasicAuthenticator>)
pub fn build_component(&self) -> (AccountComponent, Option<BasicAuthenticator>)
Converts self into its corresponding authentication AccountComponent and an optional
BasicAuthenticator. The component is always returned, but the authenticator is only
Some when Auth::BasicAuth is passed.“