pub enum Auth {
BasicAuth,
EcdsaK256KeccakAuth,
EcdsaK256KeccakAcl {
auth_trigger_procedures: Vec<Word>,
allow_unauthorized_output_notes: bool,
allow_unauthorized_input_notes: bool,
},
EcdsaK256KeccakMultisig {
threshold: u32,
approvers: Vec<Word>,
proc_threshold_map: Vec<(Word, u32)>,
},
Multisig {
threshold: u32,
approvers: Vec<Word>,
proc_threshold_map: Vec<(Word, u32)>,
},
Acl {
auth_trigger_procedures: Vec<Word>,
allow_unauthorized_output_notes: bool,
allow_unauthorized_input_notes: bool,
},
IncrNonce,
Noop,
Conditional,
}Expand description
Specifies which authentication mechanism is desired for accounts
Variants
BasicAuth
Creates a secret key for the account and creates a BasicAuthenticator used to authenticate the account with AuthRpoFalcon512.
EcdsaK256KeccakAuth
Creates a secret key for the account and creates a BasicAuthenticator used to authenticate the account with AuthEcdsaK256Keccak.
EcdsaK256KeccakAcl
Creates a secret key for the account, and creates a BasicAuthenticator used to authenticate the account with AuthEcdsaK256KeccakAcl. Authentication will only be triggered if any of the procedures specified in the list are called during execution.
EcdsaK256KeccakMultisig
Multisig
Multisig
Acl
Creates a secret key for the account, and creates a BasicAuthenticator used to authenticate the account with AuthRpoFalcon512Acl. Authentication will only be triggered if any of the procedures specified in the list are called during execution.
IncrNonce
Creates a mock authentication mechanism for the account that only increments the nonce.
Noop
Creates a mock authentication mechanism for the account that does nothing.
Conditional
Creates a mock authentication mechanism for the account that conditionally succeeds and conditionally increments the nonce based on the authentication arguments.
The auth procedure expects the first three arguments as [99, 98, 97] to succeed. In case it succeeds, it conditionally increments the nonce based on the fourth argument.
Implementations
Sourceimpl Auth
impl Auth
Sourcepub fn build_component(&self) -> (AccountComponent, Option<BasicAuthenticator>)
pub fn build_component(&self) -> (AccountComponent, Option<BasicAuthenticator>)
Converts self into its corresponding authentication AccountComponent and an optional
BasicAuthenticator. The component is always returned, but the authenticator is only
Some when Auth::BasicAuth is passed.“