pub enum Auth {
BasicAuth,
ProcedureAcl {
auth_trigger_procedures: Vec<Digest>,
},
IncrNonce,
Noop,
Conditional,
}Expand description
Specifies which authentication mechanism is desired for accounts
Variants
BasicAuth
Creates a SecretKey for the account and creates a BasicAuthenticator used to authenticate the account with RpoFalcon512.
ProcedureAcl
Creates a SecretKey for the account, and creates a BasicAuthenticator used to authenticate the account with RpoFalcon512ProcedureAcl. Authentication will only be triggered if any of the procedures specified in the list are called during execution.
IncrNonce
Creates a mock authentication mechanism for the account that only increments the nonce.
Noop
Creates a mock authentication mechanism for the account that does nothing.
Conditional
TODO update once #1501 is ready.
Implementations
Sourceimpl Auth
impl Auth
Sourcepub fn build_component(
&self,
) -> (AccountComponent, Option<BasicAuthenticator<ChaCha20Rng>>)
pub fn build_component( &self, ) -> (AccountComponent, Option<BasicAuthenticator<ChaCha20Rng>>)
Converts self into its corresponding authentication AccountComponent and an optional
BasicAuthenticator. The component is always returned, but the authenticator is only
Some when Auth::BasicAuth is passed.“