pub struct TransactionContext { /* private fields */ }Expand description
Represents all needed data for executing a transaction, or arbitrary code.
It implements DataStore, so transactions may be executed with
TransactionExecutor
Implementations
Sourceimpl TransactionContext
impl TransactionContext
Sourcepub fn execute_code(&self, code: &str) -> Result<Process, ExecutionError>
pub fn execute_code(&self, code: &str) -> Result<Process, ExecutionError>
Executes arbitrary code within the context of a mocked transaction environment and returns the resulting Process.
The code is compiled with the assembler returned by
TransactionKernel::with_mock_libraries and executed with advice inputs constructed from
the data stored in the context. The program is run on a MockHost which is loaded with
the procedures exposed by the transaction kernel, and also individual kernel functions (not
normally exposed).
To improve the error message quality, convert the returned ExecutionError into a
Report.
Errors
Returns an error if the assembly or execution of the provided code fails.
Panics
- If the provided
codeis not a valid program.
Sourcepub async fn execute(
self,
) -> Result<ExecutedTransaction, TransactionExecutorError>
pub async fn execute( self, ) -> Result<ExecutedTransaction, TransactionExecutorError>
Executes the transaction through a TransactionExecutor
Sourcepub fn execute_blocking(
self,
) -> Result<ExecutedTransaction, TransactionExecutorError>
pub fn execute_blocking( self, ) -> Result<ExecutedTransaction, TransactionExecutorError>
Executes the transaction through a TransactionExecutor
TODO: This is a temporary workaround to avoid having to update each test to use tokio::test. Eventually we should get rid of this method and use tokio::test + execute, but for the POC stage this is easier.
pub fn account(&self) -> &Account
pub fn expected_output_notes(&self) -> &[Note]
pub fn tx_args(&self) -> &TransactionArgs
pub fn input_notes(&self) -> &InputNotes<InputNote>
pub fn set_tx_args(&mut self, tx_args: TransactionArgs)
pub fn tx_inputs(&self) -> &TransactionInputs
pub fn authenticator(&self) -> Option<&BasicAuthenticator<ChaCha20Rng>>
Sourcepub fn source_manager(&self) -> Arc<dyn SourceManagerSync>
pub fn source_manager(&self) -> Arc<dyn SourceManagerSync>
Returns the source manager used in the assembler of the transaction context builder.